@@ -609,7 +609,7 @@ spec = do
           [ intercalate
               "\n"
               [ "[DEBUG]: Applied 'COPY' (28 nodes -> 25 nodes)"
-              , "{⟦"
+              , "⟦"
               , "  x ↦ ⟦"
               , "    y ↦ 5"
               , "---| log is limited by --log-lines=4 option |---"
@@ -622,6 +622,12 @@ spec = do
           ["rewrite", "--canonize", "--sweet", "--flat"]
           ["{⟦ x ↦ ⟦ y ↦ ⟦ λ ⤍ F1 ⟧.q, z ↦ Φ.x( a ↦ ⟦ w ↦ ⟦ λ ⤍ F2 ⟧, λ ⤍ F3 ⟧ ) ⟧, λ ⤍ F4 ⟧}"]
 
+    it "rewrites by locator" $
+      withStdin "{[[ ex -> [[ x -> [[ y -> 5 ]].y ]], abc -> [[ x -> ? ]](x -> 5) ]]}" $
+        testCLISucceeded
+          ["rewrite", "--sweet", "--flat", "--locator=Q.ex", "--normalize"]
+          ["{⟦ ex ↦ ⟦ x ↦ 5 ⟧, abc ↦ ⟦ x ↦ ∅ ⟧( x ↦ 5 ) ⟧}"]
+
   describe "dataize" $ do
     it "prints help" $
       testCLISucceeded ["dataize", "--help"] ["Dataize the 𝜑-program"]
